library vehicle_keyboard;
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

/// 车牌输入键盘
class VehicleKeyboard extends StatefulWidget {
  final callback;

  TextEditingController controller;

  VehicleKeyboard(this.callback,this.controller);

  @override
  State<StatefulWidget> createState() => VehicleKeyboardState();
}

class VehicleKeyboardState extends State<VehicleKeyboard> {

  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();
  bool isFirst = true;

  final double maxHeight = ScreenUtil.instance.setHeight(680);
  final double minHeight = ScreenUtil.instance.setHeight(580);

  final double textSize = 20;

  double height = ScreenUtil.instance.setHeight(680);

  double keyWidth = ScreenUtil.instance.setHeight(100);

  /// 键盘的整体回调，根据不同的按钮事件来进行相应的逻辑实现
  void _onKeyDown(KeyDownEvent data) {
    debugPrint("pressKey:" + data.key);

    if (data.isDelete()) {
      widget.controller.text =
          widget.controller.text.substring(0, widget.controller.text.length - 1);
      switchContent(widget.controller);
    } else if (data.isMore()) {
      setState(() {
        height = maxHeight;
        isFirst = false;
      });

    } else if (data.isProvince()) {
      setState(() {
        isFirst = true;
        height = minHeight;
      });
    } else {
      widget.controller.text += data.key;
      switchContent(widget.controller);
    }
  }


  void switchContent(TextEditingController controller) {
    setState(() {
      if (controller.text.length > 0) {
        isFirst = false;
        height = maxHeight;
      } else {
        isFirst = true;
        height = minHeight;
      }
    });
  }

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    /* ScreenUtil.instance = ScreenUtil(width: 1080, height: 1920)..init(context);

    print(ScreenUtil.instance.width);
    print(ScreenUtil.instance.height);
    keyWidth = ScreenUtil.instance.setWidth(80);*/
    switchContent(widget.controller);
  }
  @override
  Widget build(BuildContext context) {
    //设置适配尺寸 (填入设计稿中设备的屏幕尺寸)
    ScreenUtil.instance = ScreenUtil(width: 1080, height: 1920)..init(context);
    print(ScreenUtil.instance.width);
    print(ScreenUtil.instance.height);
    keyWidth = ScreenUtil.instance.setWidth(100);
    return new Container(
      color: Colors.grey[300],
      key: _scaffoldKey,
      width: ScreenUtil.instance.width,
      height: height,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        children: <Widget>[
          Divider(
            height: 0.8,
            indent: 0.0,
            color: Colors.grey,
          ),
          Container(
            width: ScreenUtil.instance.width,
            height: ScreenUtil.instance.setHeight(75),
            color: Colors.white,
            child: Row(
              children: <Widget>[
                Expanded(
                  flex: 1,
                  child: Container(
                    color: Colors.white,
                    /* child: TextField(
                      onChanged: (value) {},
                    ),*/
                  ),
                ),
                Container(
                    height: ScreenUtil.instance.setHeight(75),
                    color: Colors.white,
                    child: Center(
                      child: IconButton(
                        icon: Icon(
                          Icons.arrow_drop_down,
                        ),
                        alignment: AlignmentDirectional.center,
                        onPressed: () {
                          widget.callback(new KeyDownEvent('close'));
                        },
                      ),
                    ))
              ],
            ),
          ),
          isFirst ? _province() : _otherKey(),
          SizedBox(height: ScreenUtil.bottomBarHeight,)

        ],
      ),
    );
  }

  /// 键盘按键
  Widget raiseButton(String name) {
    return Container(
      width: keyWidth,
      child: RaisedButton(
        child: Text(
          name,
          textAlign: TextAlign.center,
          style: TextStyle(fontSize: textSize, color: Colors.black),
        ),
        color: Colors.white,
        textTheme: ButtonTextTheme.accent,
        textColor: Colors.black,
        shape: RoundedRectangleBorder(
            borderRadius: BorderRadiusDirectional.circular(5)),
        onPressed: () {
          _onKeyDown(new KeyDownEvent(name));
          widget.callback(new KeyDownEvent(name));
        },
      ),
    );
  }

  /// 确定按键
  Widget commitButton(bool enable) {
    return Container(
      width: ScreenUtil.instance.setWidth(230),
      child: RaisedButton(
        child: Text(
          '确定',
          textAlign: TextAlign.center,
          style: TextStyle(fontSize: textSize, color: Colors.white),
        ),
        color: Colors.lightBlue,
        disabledColor: Colors.grey[400],
        textTheme: ButtonTextTheme.accent,
        textColor: Colors.black,
        shape: RoundedRectangleBorder(
            borderRadius: BorderRadiusDirectional.circular(5)),
        onPressed: enable
            ? () {
          widget.callback(new KeyDownEvent('commit'));
        }
            : null,
      ),
    );
  }

  /// 删除按键
  Widget delButton() {
    return Container(
      width: ScreenUtil.instance.setWidth(200),
      child: RaisedButton(
        child: Icon(Icons.backspace),
        color: Colors.white,
        shape: RoundedRectangleBorder(
            borderRadius: BorderRadiusDirectional.circular(5)),
        onPressed: () {
          _onKeyDown(new KeyDownEvent('del'));
          widget.callback(new KeyDownEvent('del'));
        },
      ),
    );
  }


  /// 切换按键
  Widget switchButton(String name, String key) {
    return Container(
      width: ScreenUtil.instance.setWidth(230),
      child: RaisedButton(
          child: Text(
            name,
            textAlign: TextAlign.center,
            style: TextStyle(fontSize: textSize, color: Colors.black),
          ),
          color: Colors.white,
          textTheme: ButtonTextTheme.accent,
          textColor: Colors.black,
          shape: RoundedRectangleBorder(
              borderRadius: BorderRadiusDirectional.circular(5)),
          onPressed: () {
            _onKeyDown(new KeyDownEvent(key));
            widget.callback(new KeyDownEvent(key));
          }
      ),
    );
  }

  /// 车牌首字符
  Widget _province() {
    return Container(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        children: <Widget>[
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              raiseButton('皖'),
              raiseButton('京'),
              raiseButton('津'),
              raiseButton('冀'),
              raiseButton('晋'),
              raiseButton('辽'),
              raiseButton('吉'),
              raiseButton('黑'),
              raiseButton('沪'),
              raiseButton('苏'),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              raiseButton('浙'),
              raiseButton('渝'),
              raiseButton('闽'),
              raiseButton('赣'),
              raiseButton('鲁'),
              raiseButton('豫'),
              raiseButton('鄂'),
              raiseButton('湘'),
              raiseButton('粤'),
              raiseButton('琼'),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              raiseButton('川'),
              raiseButton('贵'),
              raiseButton('云'),
              raiseButton('陕'),
              raiseButton('甘'),
              raiseButton('青'),
              raiseButton('蒙'),
              raiseButton('桂'),
              delButton(),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              raiseButton('宁'),
              raiseButton('新'),
              raiseButton('藏'),
              SizedBox(width: ScreenUtil.instance.setWidth(100)),
              switchButton('更多', 'more'),
              commitButton(false),
            ],
          )
        ],
      ),
    );
  }

  /// 其他字符
  Widget _otherKey() {
    return Container(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              raiseButton('1'),
              raiseButton('2'),
              raiseButton('3'),
              raiseButton('4'),
              raiseButton('5'),
              raiseButton('6'),
              raiseButton('7'),
              raiseButton('8'),
              raiseButton('9'),
              raiseButton('0'),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              raiseButton('A'),
              raiseButton('B'),
              raiseButton('C'),
              raiseButton('D'),
              raiseButton('E'),
              raiseButton('F'),
              raiseButton('G'),
              raiseButton('H'),
              raiseButton('J'),
              raiseButton('K'),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              raiseButton('L'),
              raiseButton('M'),
              raiseButton('N'),
              raiseButton('P'),
              raiseButton('Q'),
              raiseButton('R'),
              raiseButton('S'),
              raiseButton('T'),
              raiseButton('U'),
              raiseButton('v'),

            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              raiseButton('W'),
              raiseButton('X'),
              raiseButton('Y'),
              raiseButton('Z'),
              raiseButton('警'),
              raiseButton('学'),
              raiseButton('挂'),
              raiseButton('超'),
              delButton(),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              raiseButton('使'),
              raiseButton('领'),
              raiseButton('港'),
              raiseButton('澳'),
              SizedBox(width: ScreenUtil.instance.setWidth(80),),
              switchButton('省份', 'province'),
              commitButton(true),
            ],
          )
        ],
      ),
    );
  }
}

/// 点击事件
class KeyDownEvent {
  //当前点击的按钮所代表的值
  String key;

  KeyDownEvent(this.key);

  bool isDelete() => this.key == "del";

  bool isMore() => this.key == "more";

  bool isProvince() => this.key == "province";

  bool isClose() => this.key == "close";

  bool isCommit() => this.key == "commit";
}
